"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = __importDefault(require("node-fetch"));
const entropic_bond_1 = require("entropic-bond");
const firebase_datasource_1 = require("./firebase-datasource");
const firebase_helper_1 = require("../firebase-helper");
const test_user_1 = require("../mocks/test-user");
const mock_data_json_1 = __importDefault(require("../mocks/mock-data.json"));
function loadTestData(model) {
    return __awaiter(this, void 0, void 0, function* () {
        const users = mock_data_json_1.default.TestUser;
        const promises = [];
        for (const key in users) {
            const user = entropic_bond_1.Persistent.createInstance(users[key]);
            promises.push(model.save(user));
        }
        yield Promise.all(promises);
    });
}
describe('Firestore Model', () => {
    let model;
    let testUser;
    firebase_helper_1.FirebaseHelper.setFirebaseConfig({
        projectId: "demo-test",
    });
    firebase_helper_1.FirebaseHelper.useEmulator({ firestorePort: 9080 });
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        entropic_bond_1.Store.useDataSource(new firebase_datasource_1.FirebaseDatasource());
        testUser = new test_user_1.TestUser();
        testUser.name = {
            firstName: 'testUserFirstName',
            lastName: 'testUserLastName'
        };
        testUser.age = 35;
        testUser.skills = ['lazy', 'dirty'];
        model = entropic_bond_1.Store.getModel('TestUser');
        yield loadTestData(model);
    }));
    afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
        firebase_helper_1.FirebaseHelper.instance.firestore().terminate();
        yield node_fetch_1.default('http://localhost:9080/emulator/v1/projects/demo-test/databases/(default)/documents', {
            method: 'DELETE'
        });
    }));
    it('should find document by id', () => __awaiter(void 0, void 0, void 0, function* () {
        yield model.save(testUser);
        const user = yield model.findById(testUser.id);
        expect(user).toBeInstanceOf(test_user_1.TestUser);
        expect(user.id).toEqual(testUser.id);
        expect(user.name.firstName).toEqual('testUserFirstName');
    }));
    it('should write a document', () => __awaiter(void 0, void 0, void 0, function* () {
        yield model.save(testUser);
        const newUser = yield model.findById(testUser.id);
        expect(newUser.name).toEqual({
            firstName: 'testUserFirstName',
            lastName: 'testUserLastName'
        });
    }));
    it('should delete a document by id', () => __awaiter(void 0, void 0, void 0, function* () {
        yield model.save(testUser);
        const newUser = yield model.findById(testUser.id);
        expect(newUser.age).toBe(35);
        yield model.delete(testUser.id);
        const deletedUser = yield model.findById(testUser.id);
        expect(deletedUser).toBeUndefined();
    }));
    it('should not throw if a document id doesn\'t exists', (done) => {
        expect(() => {
            model.findById('nonExistingId')
                .then(done)
                .catch(done);
        }).not.toThrow();
    });
    it('should return undefined if a document id doesn\'t exists', () => __awaiter(void 0, void 0, void 0, function* () {
        expect(yield model.findById('nonExistingId')).toBeUndefined();
    }));
    it('should retrieve array fields', () => __awaiter(void 0, void 0, void 0, function* () {
        yield model.save(testUser);
        const newUser = yield model.findById(testUser.id);
        expect(Array.isArray(newUser.skills)).toBeTruthy();
        expect(newUser.skills).toEqual(expect.arrayContaining(['lazy', 'dirty']));
    }));
    it('should retrieve object fields', () => __awaiter(void 0, void 0, void 0, function* () {
        yield model.save(testUser);
        const newUser = yield model.findById(testUser.id);
        expect(newUser.name).toEqual({
            firstName: 'testUserFirstName',
            lastName: 'testUserLastName'
        });
    }));
    describe('Generic find', () => {
        it('should query all admins with query object', () => __awaiter(void 0, void 0, void 0, function* () {
            testUser.admin = true;
            yield model.save(testUser);
            const admins = yield model.query({
                operations: [{
                        property: 'admin',
                        operator: '==',
                        value: true
                    }]
            });
            expect(admins.length).toBeGreaterThanOrEqual(1);
            expect(admins[0]).toBeInstanceOf(test_user_1.TestUser);
        }));
        it('should find all admins with where methods', () => __awaiter(void 0, void 0, void 0, function* () {
            const admins = yield model.find().where('admin', '==', true).get();
            expect(admins.length).toBeGreaterThanOrEqual(1);
            expect(admins[0]).toBeInstanceOf(test_user_1.TestUser);
        }));
        it('should find admins with age less than 56', () => __awaiter(void 0, void 0, void 0, function* () {
            const admins = yield model.find()
                .where('admin', '==', true)
                .where('age', '<', 50)
                .get();
            expect(admins.length).toBeGreaterThanOrEqual(1);
            expect(admins[0].age).toBeLessThan(50);
        }));
        it('should query by subproperties', () => __awaiter(void 0, void 0, void 0, function* () {
            const users = yield model.query({
                operations: [
                    {
                        property: 'name',
                        operator: '==',
                        value: { firstName: 'userFirstName3' }
                    },
                    { property: 'age', operator: '!=', value: 134 }
                ]
            });
            expect(users[0].id).toBe('user3');
        }));
        it('should find by subproperties', () => __awaiter(void 0, void 0, void 0, function* () {
            const users = yield model.find()
                .where('name', '==', { firstName: 'userFirstName3' })
                .get();
            expect(users[0].id).toBe('user3');
        }));
        it('should find by property path', () => __awaiter(void 0, void 0, void 0, function* () {
            const users = yield model.find()
                .whereDeepProp('name.firstName', '==', 'userFirstName3')
                .get();
            expect(users[0].id).toBe('user3');
        }));
        it('should find by superdeep property path', () => __awaiter(void 0, void 0, void 0, function* () {
            const users = yield model.find()
                .whereDeepProp('name.ancestorName.father', '==', 'user3Father')
                .get();
            expect(users[0].id).toEqual('user3');
        }));
        it('should find by swallow property path', () => __awaiter(void 0, void 0, void 0, function* () {
            const users = yield model.find()
                .whereDeepProp('age', '==', 21)
                .get();
            expect(users[0].id).toEqual('user2');
        }));
    });
    describe('Derived classes should fit on parent collection', () => {
        it('should save derived object in parent collection', () => __awaiter(void 0, void 0, void 0, function* () {
            const derived = new test_user_1.DerivedUser();
            derived.name = { firstName: 'Fulanito', lastName: 'Derived' };
            derived.salary = 3900;
            yield model.save(derived);
            const newUser = yield model.findById(derived.id);
            expect(newUser).toBeInstanceOf(test_user_1.DerivedUser);
            expect(newUser.salary).toBe(3900);
            expect(newUser.className).toEqual('DerivedUser');
        }));
    });
    describe('References to documents', () => {
        let ref1, ref2;
        beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
            testUser.documentRef = new test_user_1.SubClass();
            testUser.documentRef.year = 2045;
            ref1 = new test_user_1.SubClass();
            ref1.year = 2081;
            ref2 = new test_user_1.SubClass();
            ref2.year = 2082;
            testUser.manyRefs.push(ref1);
            testUser.manyRefs.push(ref2);
            testUser.derived = new test_user_1.DerivedUser();
            testUser.derived.salary = 1350;
            testUser.manyDerived = [new test_user_1.DerivedUser(), new test_user_1.DerivedUser()];
            testUser.manyDerived[0].salary = 990;
            testUser.manyDerived[1].salary = 1990;
            yield model.save(testUser);
        }));
        it('should save a document as a reference', () => __awaiter(void 0, void 0, void 0, function* () {
            const subClassModel = entropic_bond_1.Store.getModel('SubClass');
            expect(subClassModel).toBeDefined();
            const newDocument = yield subClassModel.findById(testUser.documentRef.id);
            expect(newDocument).toBeInstanceOf(test_user_1.SubClass);
            expect(newDocument.year).toBe(2045);
        }));
        it('should read a swallow document reference', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById(testUser.id);
            expect(loadedUser.documentRef).toBeInstanceOf(test_user_1.SubClass);
            expect(loadedUser.documentRef.id).toBeDefined();
            expect(loadedUser.documentRef.year).toBeUndefined();
        }));
        it('should fill data of swallow document reference', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById(testUser.id);
            yield entropic_bond_1.Store.populate(loadedUser.documentRef);
            expect(loadedUser.documentRef.id).toBeDefined();
            expect(loadedUser.documentRef.year).toBe(2045);
        }));
        it('should save and array of references', () => __awaiter(void 0, void 0, void 0, function* () {
            const subClassModel = entropic_bond_1.Store.getModel('SubClass');
            const newDocument = yield subClassModel.findById(testUser.documentRef.id);
            expect(newDocument).toBeInstanceOf(test_user_1.SubClass);
            expect(newDocument.year).toBe(2045);
        }));
        it('should read an array of references', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById(testUser.id);
            expect(loadedUser.manyRefs).toHaveLength(2);
            expect(loadedUser.manyRefs[0]).toBeInstanceOf(test_user_1.SubClass);
            expect(loadedUser.manyRefs[0].id).toEqual(testUser.manyRefs[0].id);
            expect(loadedUser.manyRefs[0].year).toBeUndefined();
            expect(loadedUser.manyRefs[1]).toBeInstanceOf(test_user_1.SubClass);
            expect(loadedUser.manyRefs[1].id).toEqual(testUser.manyRefs[1].id);
            expect(loadedUser.manyRefs[1].year).toBeUndefined();
        }));
        it('should fill array of refs', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById(testUser.id);
            yield entropic_bond_1.Store.populate(loadedUser.manyRefs);
            expect(loadedUser.manyRefs[0].year).toBe(2081);
            expect(loadedUser.manyRefs[1].year).toBe(2082);
        }));
        it('should save a reference when declared @persistentAt', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById(testUser.id);
            expect(loadedUser.derived.id).toEqual(testUser.derived.id);
            expect(loadedUser.derived.salary).toBeUndefined();
            yield entropic_bond_1.Store.populate(loadedUser.derived);
            expect(loadedUser.derived.salary).toBe(1350);
            expect(loadedUser.derived.id).toBe(testUser.derived.id);
        }));
        it('should populate from special collection when declared with @persistentRefAt', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById('user6');
            yield entropic_bond_1.Store.populate(loadedUser.derived);
            expect(loadedUser.derived.salary).toBe(2800);
            expect(loadedUser.derived.id).toBe('user4');
        }));
        it('should save a reference when declared @persistentAt as array', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById(testUser.id);
            expect(loadedUser.manyDerived[0].id).toEqual(testUser.manyDerived[0].id);
            expect(loadedUser.manyDerived[0].salary).toBeUndefined();
            expect(loadedUser.manyDerived[1].salary).toBeUndefined();
            yield entropic_bond_1.Store.populate(loadedUser.manyDerived);
            expect(loadedUser.manyDerived[0].salary).toBe(990);
            expect(loadedUser.manyDerived[0].id).toBe(testUser.manyDerived[0].id);
            expect(loadedUser.manyDerived[1].salary).toBe(1990);
            expect(loadedUser.manyDerived[1].id).toBe(testUser.manyDerived[1].id);
        }));
        it('should not overwrite not filled ref in collection', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById('user6');
            yield model.save(loadedUser);
            const refInCollection = yield model.findById('user4');
            expect(refInCollection.salary).toBe(2800);
        }));
        it('should save loaded ref with assigned new instance', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById('user6');
            loadedUser.derived = new test_user_1.DerivedUser();
            loadedUser.derived.salary = 345;
            yield model.save(loadedUser);
            const refInCollection = yield model.findById(loadedUser.derived.id);
            expect(refInCollection.salary).toBe(345);
        }));
        it('should save loaded ref with modified ref data', () => __awaiter(void 0, void 0, void 0, function* () {
            const loadedUser = yield model.findById('user6');
            yield entropic_bond_1.Store.populate(loadedUser.derived);
            loadedUser.derived.salary = 1623;
            yield model.save(loadedUser);
            const refInCollection = yield model.findById('user4');
            expect(refInCollection.salary).toBe(1623);
        }));
    });
    describe('Operations on queries', () => {
        it('should limit the result set', () => __awaiter(void 0, void 0, void 0, function* () {
            const unlimited = yield model.find().get();
            const limited = yield model.find().limit(2).get();
            expect(unlimited.length).not.toBe(limited.length);
            expect(limited).toHaveLength(2);
        }));
        it('should sort ascending the result set', () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = yield model.find().orderBy('age').get();
            expect(docs[0].id).toEqual('user2');
            expect(docs[1].id).toEqual('user1');
        }));
        it('should sort descending the result set', () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = yield model.find().orderBy('age', 'desc').get();
            expect(docs[0].id).toEqual('user3');
            expect(docs[1].id).toEqual('user5');
        }));
        it('should sort by deep property path', () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = yield model.find().orderByDeepProp('name.firstName', 'desc').get();
            expect(docs[0].id).toEqual('user6');
            expect(docs[1].id).toEqual('user5');
        }));
        it('should sort by swallow property path', () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = yield model.find().orderByDeepProp('age').get();
            expect(docs[0].id).toEqual('user2');
            expect(docs[1].id).toEqual('user1');
        }));
        describe('Data Cursors', () => {
            beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
                yield model.find().get(2);
            }));
            it('should get next result set', () => __awaiter(void 0, void 0, void 0, function* () {
                const docs = yield model.next();
                const mockDataArr = Object.values(mock_data_json_1.default.TestUser);
                expect(docs).toHaveLength(2);
                expect(docs[0].id).toEqual(mockDataArr[2].id);
                expect(docs[0].id).toEqual('user3');
            }));
            it('should not go beyond the end of result set', () => __awaiter(void 0, void 0, void 0, function* () {
                yield model.next();
                yield model.next();
                const docs = yield model.next();
                expect(docs).toHaveLength(0);
            }));
        });
    });
});
//# sourceMappingURL=firebase-datasource.spec.js.map